// ----------------------------------
// setup command line args
// @see https://www.npmjs.com/package/commander
// ----------------------------------
const commander = require('commander');
const yaml = require('js-yaml');

// read a yaml or die
function getYamlFile(yamlFilename) {
    return yaml.safeLoad(fs.readFileSync(yamlFilename, 'utf8'));
}

commander
  .version('0.0.3')
  .option('-c, --config [path]', 'Config file [config.yaml]', 'config.yaml')
  .option('-y, --deploy [path]', 'Deployment file')
  .option('-a, --data [path]', 'Preset Data file')
  .option('-d, --api-debug', 'API debug trace')
  .parse(process.argv);

console.log('command line:', commander.rawArgs);
if (commander.config === undefined) {
  console.log('No config file defined (use --config)');
  process.exit();
}
console.log('config filename:', commander.config);
const configFile = {
  "apiDebug": true,
  "password": "1234",
  "timeout": 600000,
  "libPath": "./server/lib",
  "dataFilename": "./server/dapp/dapp.presets.yaml",
  "deployFilename": "./server/config/server.deploy.yaml",
  "nodes": [
    {
      "id": 0,
      "explorerUrl": "http://blockxorhs.eastus.cloudapp.azure.com:9000",
      "stratoUrl": "http://blockxorhs.eastus.cloudapp.azure.com/strato-api",
      "blocUrl": "http://blockxorhs.eastus.cloudapp.azure.com/bloc/v2.2",
      "searchUrl": "http://blockxorhs.eastus.cloudapp.azure.com/bloc/v2.2",
//      "searchUrl": "http://blockxorhs.eastus.cloudapp.azure.com/cirrus"
    }
  ]
};
// cli overrides config
if (commander.apiDebug) configFile.apiDebug = true;
if (commander.timeout !== undefined) configFile.timeout = commander.timeout;
if (commander.data !== undefined) configFile.data = commander.data;
console.log('config', JSON.stringify(configFile, null, 2));
configFile.apiDebug = true;
exports.configFile = configFile;

// deploy
if (commander.deploy) configFile.deploy = commander.deploy;

// api urls
configFile.getBlocUrl = function (node) {
  if (node === undefined) node = 0;
  return this.nodes[node].blocUrl;
}
configFile.getExplorerUrl = function (node) {
  if (node === undefined) node = 0;
  return this.nodes[node].explorerUrl;
}
configFile.getStratoUrl = function (node) {
  if (node === undefined) node = 0;
  return this.nodes[node].stratoUrl;
}
configFile.getSearchUrl = function (node) {
  if (node === undefined) node = 0;
  return this.nodes[node].searchUrl;
}
